/*
	General Array Class

	Handles a growable array. Accessing the 'n'th element will grow the
	array to at least that point (usually furthur).
	
	Author: Matthew Allen <fret@memecode.com>
	Date: 5/12/2003
*/

#ifndef _ARRAY_H_
#define _ARRAY_H_

template <class Type>
class Array
{
	Type *p;
	int len;
	int alloc;

public:
	Array()
	{
		p = 0;
		len = 0;
		alloc = 0;
	}
	
	~Array()
	{
		DeleteArray(p);
	}	
	
	int Length()
	{
		return len;
	}
	
	Type &operator [](int i)
	{
		// check array bounds
		assert(i >= 0);
		
		if (i >= alloc)
		{
			// increase array length
			int nalloc = alloc ? alloc : 0x100;
			while (nalloc <= i)
				nalloc <<= 1;

			// alloc new array
			Type *np = new Type[nalloc];
			if (np)
			{
				// clear new cells
				memset(np + len, 0, (nalloc - len) * sizeof(Type));
				if (p)
				{
					// copy across old cells
					memcpy(np, p, len * sizeof(Type));
					
					// clear old array
					delete [] p;
				}
				
				// new values
				p = np;
				alloc = nalloc;
			}
			else
			{
				static Type *t = 0;
				return *t;
			}
		}

		// adjust length of the the array
		if (i + 1 > len)
		{
			len = i + 1;
		}
		
		return p[i];
	}
};

#endif
