/*
	Private libsharedmime header

	This is all the private definitions local to my implementation of
	libsharedmime. Some of which is taken from LGI
	(http://www.memecode.com/lgi.php)
	
	Author: Matthew Allen <fret@memecode.com>
	Date: 5/12/2003
*/

#include "mime-types-priv.h"
#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

int Lex(Array<char*> &Tokens, char *Str, char *Delim)
{
	if (Str && Delim)
	{
		bool Lut[256];
		memset(Lut, 0, sizeof(Lut));
		for (char *d = Delim; *d; d++)
		{
			Lut[(uchar)*d] = true;
		}
		
		for (char *s = Str; *s;)
		{
			// skip delim
			while (*s && Lut[*s]) s++;
			
			// set start
			Tokens[Tokens.Length()] = s;
			
			// skip non-delim
			while (*s && !Lut[*s]) s++;
			
			// set end
			if (*s)
			{
				*s++ = 0;
			}
			else break;			
		}
	}
	
	return Tokens.Length();
}

void TrimDir(char *Path)
{
	char *d = strrchr(Path, DIR_CHAR);
	if (d)
	{
		*d = 0;
		if (d[1])
		{
			d = strrchr(Path, DIR_CHAR);
			if (d)
			{
				*d = 0;
			}
		}
	}
}

bool MakePath(char *Str, char *Path, char *File)
{
	if (Str && Path && File)
	{
		if (Str != Path)
		{
			strcpy(Str, Path);
		}
		
		if (Str[0] == '~')
		{
			// expand the home dir
			struct passwd *pw = getpwuid(getuid());
			if (pw)
			{
				int Len = strlen(Str);
				int Home = strlen(pw->pw_dir);
				memmove(Str + Home, Str + 1, Len);
				memcpy(Str, pw->pw_dir, Home);
			}
			else return false;
		}

		char Dir[] = { '/', '\\', 0 };
		#define EndStr() Str[strlen(Str)-1]
		#define EndDir() if (! strchr(Dir, EndStr())) strcat(Str, DIR_STR);

		char *End = Str + strlen(Str) - 1;
		if (strchr(Dir, *End))
		{
			*End = 0;
		}

		Array<char*> T;
		if (Lex(T, File, Dir))
		{
			for (int i=0; i<T.Length(); i++)
			{
				if (stricmp(T[i], ".") == 0)
				{
				}
				else if (stricmp(T[i], "..") == 0)
				{
					TrimDir(Str);
				}
				else
				{
					EndDir();
					strcat(Str, T[i]);
				}
			}
		}
		
		return true;
	}
	
	return false;
}

char *ReadTextFile(char *File)
{
	char *Mem = 0;
	
	struct stat s;
	if (stat(File, &s) == 0)
	{
	    FILE *f = fopen(File, "rb");
		if (f)
		{
			Mem = new char[s.st_size+1];
			if (Mem)
			{
				fread(Mem, 1, s.st_size, f);
				Mem[s.st_size] = 0;
			}
			
			fclose(f);
		}
	}

	return Mem;
}

char *NewStr(char *Str, int Len)
{
	char *Mem = 0;

	if (Str)
	{
		if (Len < 0)
		{
			Len = strlen(Str);
		}
		
		Mem = new char[Len + 1];
		if (Mem)
		{
			memcpy(Mem, Str, Len);
			Mem[Len] = 0;
		}
	}
	
	return Mem;
}

bool MatchStr(	char *a, // the template
				char *b  // the data being tested
				)
{
	if (!a)
	{
		return false;
	}

	if (stricmp(a, "*") == 0)
	{
		// matches anything
		return true;
	}

	if (!b)
	{
		return false;
	}

	while (*a && *b)
	{
		if (*a == '*')
		{
			a++;

			if (*a)
			{
				char *EndA;
				for (EndA = a; *EndA && *EndA!='?' && *EndA!='*'; EndA++);
				int SegLen = (int)EndA-(int)a;
				char *Seg = NewStr(a, SegLen);
				if (!Seg) return false;

				// find end of non match
				while (*b)
				{
					if (strnicmp(b, Seg, SegLen) == 0)
					{
						break;
					}
					b++;
				}

				DeleteArray(Seg);

				if (*b)
				{
					a += SegLen;
					b += SegLen;
				}
				else
				{
					// can't find any matching part in b
					return false;
				}
			}
			else
			{
				// '*' matches everything
				return true;
			}

		}
		else if (*a == '?')
		{
			a++;
			b++;
		}
		else
		{
			if (tolower(*a) != tolower(*b))
			{
				return false;
			}

			a++;
			b++;
		}
	}

	return	((*a == 0) || (stricmp(a, "*") == 0)) &&
			(*b == 0);
}

